=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-09-17 12:46:56 +0000
+++ bzrlib/builtins.py	2007-11-27 19:37:36 +0000
@@ -4174,7 +4174,9 @@
                      value_switches=True, enum_switch=False,
                      branch='Reconfigure to a branch.',
                      tree='Reconfigure to a tree.',
-                     checkout='Reconfigure to a checkout.'),
+                     checkout='Reconfigure to a checkout.',
+                     lightweight_checkout='Reconfigure to a lightweight'
+                     ' checkout.'),
                      Option('bind-to', help='Branch to bind checkout to.',
                             type=str),
                      Option('force',
@@ -4193,6 +4195,9 @@
         elif target_type == 'checkout':
             reconfiguration = reconfigure.Reconfigure.to_checkout(directory,
                                                                   bind_to)
+        elif target_type == 'lightweight-checkout':
+            reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+                directory, bind_to)
         reconfiguration.apply(force)
 
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-09-17 12:46:56 +0000
+++ bzrlib/bzrdir.py	2007-11-27 19:37:36 +0000
@@ -236,6 +236,10 @@
             format = BzrDirFormat.get_default_format()
         return format.initialize(base, possible_transports)
 
+    def destroy_repository(self):
+        """Destroy the repository in this BzrDir"""
+        raise NotImplementedError(self.destroy_repository)
+
     def create_branch(self):
         """Create a branch in this BzrDir.
 
@@ -908,6 +912,10 @@
             raise errors.IncompatibleFormat('shared repository', self._format)
         return self.open_repository()
 
+    def destroy_repository(self):
+        """See BzrDir.destroy_repository."""
+        raise errors.UnsupportedOperation(self.destroy_repository, self)
+
     def create_workingtree(self, revision_id=None):
         """See BzrDir.create_workingtree."""
         # this looks buggy but is not -really-
@@ -1086,6 +1094,10 @@
         """See BzrDir.create_repository."""
         return self._format.repository_format.initialize(self, shared)
 
+    def destroy_repository(self):
+        """See BzrDir.destroy_repository."""
+        self.transport.delete_tree('repository')
+
     def create_workingtree(self, revision_id=None):
         """See BzrDir.create_workingtree."""
         from bzrlib.workingtree import WorkingTreeFormat

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-09-13 01:54:49 +0000
+++ bzrlib/errors.py	2007-11-27 19:37:36 +0000
@@ -2430,6 +2430,11 @@
     _fmt = "'%(display_url)s' is already a checkout."
 
 
+class AlreadyLightweightCheckout(BzrDirError):
+
+    _fmt = "'%(display_url)s' is already a lightweight checkout."
+
+
 class ReconfigurationNotSupported(BzrDirError):
 
     _fmt = "Requested reconfiguration of '%(display_url)s' is not supported."

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2007-09-14 14:45:55 +0000
+++ bzrlib/reconfigure.py	2007-11-27 19:37:36 +0000
@@ -48,10 +48,13 @@
         self._unbind = False
         self._bind = False
         self._destroy_reference = False
+        self._create_reference = False
+        self._destroy_branch = False
         self._create_branch = False
         self._destroy_tree = False
         self._create_tree = False
         self._create_repository = False
+        self._destroy_repository = False
 
     @staticmethod
     def to_branch(bzrdir):
@@ -64,7 +67,7 @@
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=False, want_branch=True,
-                                      want_bound=False)
+                                      want_bound=False, want_reference=False)
         if not reconfiguration.changes_planned():
             raise errors.AlreadyBranch(bzrdir)
         return reconfiguration
@@ -80,7 +83,7 @@
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
-                                      want_bound=False)
+                                      want_bound=False, want_reference=False)
         if not reconfiguration.changes_planned():
             raise errors.AlreadyTree(bzrdir)
         return reconfiguration
@@ -97,24 +100,58 @@
         """
         reconfiguration = Reconfigure(bzrdir, bound_location)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
-                                      want_bound=True)
+                                      want_bound=True, want_reference=False)
         if not reconfiguration.changes_planned():
             raise errors.AlreadyCheckout(bzrdir)
         return reconfiguration
 
-    def _plan_changes(self, want_tree, want_branch, want_bound):
+    @classmethod
+    def to_lightweight_checkout(klass, bzrdir, reference_location=None):
+        """Make a Reconfiguration to convert bzrdir into a lightweight checkout
+
+        :param bzrdir: The bzrdir to reconfigure
+        :param bound_location: The location the checkout should be bound to.
+        :raise errors.AlreadyLightweightCheckout: if bzrdir is already a
+            lightweight checkout
+        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
+            a branch or branch reference
+        """
+        reconfiguration = klass(bzrdir, reference_location)
+        reconfiguration._plan_changes(want_tree=True, want_branch=False,
+                                      want_bound=False, want_reference=True)
+        if not reconfiguration.changes_planned():
+            raise errors.AlreadyLightweightCheckout(bzrdir)
+        return reconfiguration
+
+    def _plan_changes(self, want_tree, want_branch, want_bound,
+                      want_reference):
         """Determine which changes are needed to assume the configuration"""
+        if not want_branch and not want_reference:
+            raise errors.ReconfigurationNotSupported(self.bzrdir)
+        if want_branch and want_reference:
+            raise errors.ReconfigurationNotSupported(self.bzrdir)
+        if (want_branch or want_reference) and (self.local_branch is None and
+                                                self.referenced_branch
+                                                is None):
+            raise errors.ReconfigurationNotSupported(self.bzrdir)
         if self.repository is None:
-            self._create_repository = True
+            if not want_reference:
+                self._create_repository = True
+        else:
+            if want_reference and (self.repository.bzrdir.root_transport.base
+                                   == self.bzrdir.root_transport.base):
+                self._destroy_repository = True
+        if self.referenced_branch is None:
+            if want_reference:
+                self._create_reference = True
+        else:
+            if not want_reference:
+                self._destroy_reference = True
         if self.local_branch is None:
             if want_branch is True:
-                if self.referenced_branch is not None:
-                    self._destroy_reference = True
-                    self._create_branch = True
-                    if want_bound:
-                        self._bind = True
-                else:
-                    raise errors.ReconfigurationNotSupported(self.bzrdir)
+                self._create_branch = True
+                if want_bound:
+                    self._bind = True
         else:
             if want_bound:
                 if self.local_branch.get_bound_location() is None:
@@ -141,18 +178,22 @@
                 raise errors.UncommittedChanges(self.tree)
 
     def _select_bind_location(self):
-        """Select a location to bind to.
+        """Select a location to bind or create a reference to.
 
         Preference is:
         1. user specified location
         2. branch reference location (it's a kind of bind location)
-        3. previous bind location (it was a good choice once)
-        4. push location (it's writeable, so committable)
-        5. parent location (it's pullable, so update-from-able)
+        3. current bind location
+        4. previous bind location (it was a good choice once)
+        5. push location (it's writeable, so committable)
+        6. parent location (it's pullable, so update-from-able)
         """
         if self.new_bound_location is not None:
             return self.new_bound_location
         if self.local_branch is not None:
+            bound = self.local_branch.get_bound_location()
+            if bound is not None:
+                return bound
             old_bound = self.local_branch.get_old_bound_location()
             if old_bound is not None:
                 return old_bound
@@ -188,11 +229,18 @@
         if self._destroy_reference:
             reference_info = self.referenced_branch.last_revision_info()
             self.bzrdir.destroy_branch()
+        if self._destroy_branch:
+            reference_info = self.local_branch.last_revision_info()
+            self.bzrdir.destroy_branch()
         if self._create_branch:
             local_branch = self.bzrdir.create_branch()
             local_branch.set_last_revision_info(*reference_info)
         else:
             local_branch = self.local_branch
+        if self._create_reference:
+            reference_branch = branch.Branch.open(self._select_bind_location())
+            format = branch.BranchReferenceFormat().initialize(self.bzrdir,
+                reference_branch)
         if self._destroy_tree:
             self.bzrdir.destroy_workingtree()
         if self._create_tree:
@@ -202,3 +250,7 @@
         if self._bind:
             bind_location = self._select_bind_location()
             local_branch.bind(branch.Branch.open(bind_location))
+        if self._destroy_repository:
+            if self._create_reference:
+                reference_branch.repository.fetch(self.repository)
+            self.bzrdir.destroy_repository()

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2007-09-14 14:45:55 +0000
+++ bzrlib/remote.py	2007-11-27 19:37:36 +0000
@@ -84,6 +84,11 @@
         self._real_bzrdir.create_repository(shared=shared)
         return self.open_repository()
 
+    def destroy_repository(self):
+        """See BzrDir.destroy_repository"""
+        self._ensure_real()
+        self._real_bzrdir.destroy_repository()
+
     def create_branch(self):
         self._ensure_real()
         real_branch = self._real_bzrdir.create_branch()

=== modified file 'bzrlib/tests/blackbox/test_reconfigure.py'
--- bzrlib/tests/blackbox/test_reconfigure.py	2007-09-17 12:45:54 +0000
+++ bzrlib/tests/blackbox/test_reconfigure.py	2007-11-27 19:37:36 +0000
@@ -56,3 +56,8 @@
         branch = self.make_branch('branch')
         checkout = branch.create_checkout('checkout', lightweight=True)
         self.run_bzr('reconfigure --checkout checkout')
+
+    def test_checkout_to_lightweight_checkout(self):
+        branch = self.make_branch('branch')
+        checkout = branch.create_checkout('checkout')
+        self.run_bzr('reconfigure --lightweight-checkout checkout')

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-09-13 01:54:49 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-11-27 19:37:36 +0000
@@ -179,6 +179,18 @@
         bzrdir.create_branch()
         bzrdir.open_branch()
 
+    def test_destroy_repository(self):
+        repo = self.make_repository('repository')
+        bzrdir = repo.bzrdir
+        try:
+            bzrdir.destroy_repository()
+        except (errors.UnsupportedOperation, errors.TransportNotPossible):
+            raise TestNotApplicable('Format does not support destroying'
+                                    ' repository')
+        self.assertRaises(errors.NoRepositoryPresent, bzrdir.open_repository)
+        bzrdir.create_repository()
+        bzrdir.open_repository()
+
     def test_open_workingtree_raises_no_working_tree(self):
         """BzrDir.open_workingtree() should raise NoWorkingTree (rather than
         e.g. NotLocalUrl) if there is no working tree.

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2007-09-13 01:54:20 +0000
+++ bzrlib/tests/test_reconfigure.py	2007-11-27 19:37:36 +0000
@@ -115,6 +115,9 @@
         branch.set_bound_location(None)
         self.assertEqual('bzr:old-bound',
                          reconfiguration._select_bind_location())
+        branch.set_bound_location('bzr:cur-bound')
+        self.assertEqual('bzr:cur-bound',
+                         reconfiguration._select_bind_location())
         reconfiguration.new_bound_location = 'ftp://user-specified'
         self.assertEqual('ftp://user-specified',
                          reconfiguration._select_bind_location())
@@ -148,3 +151,39 @@
         checkout = parent.create_checkout('checkout')
         self.assertRaises(errors.AlreadyCheckout,
                           reconfigure.Reconfigure.to_checkout, checkout.bzrdir)
+
+    def test_checkout_to_lightweight(self):
+        parent = self.make_branch('parent')
+        checkout = parent.create_checkout('checkout')
+        checkout.commit('test', rev_id='new-commit', local=True)
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            checkout.bzrdir)
+        reconfiguration.apply()
+        wt = checkout.bzrdir.open_workingtree()
+        self.assertTrue(parent.repository.has_same_location(
+            wt.branch.repository))
+        parent.repository.get_revision('new-commit')
+        self.assertRaises(errors.NoRepositoryPresent,
+                          checkout.bzrdir.open_repository)
+
+    def test_branch_to_lightweight_checkout(self):
+        parent = self.make_branch('parent')
+        child = parent.bzrdir.sprout('child').open_workingtree()
+        child.commit('test', rev_id='new-commit')
+        child.bzrdir.destroy_workingtree()
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            child.bzrdir)
+        reconfiguration.apply()
+        wt = child.bzrdir.open_workingtree()
+        self.assertTrue(parent.repository.has_same_location(
+            wt.branch.repository))
+        parent.repository.get_revision('new-commit')
+        self.assertRaises(errors.NoRepositoryPresent,
+                          child.bzrdir.open_repository)
+
+    def test_lightweight_checkout_to_lightweight_checkout(self):
+        parent = self.make_branch('parent')
+        checkout = parent.create_checkout('checkout', lightweight=True)
+        self.assertRaises(errors.AlreadyLightweightCheckout,
+                          reconfigure.Reconfigure.to_lightweight_checkout,
+                          checkout.bzrdir)

